/*
    Copyright (c) 2020 Rasmus Thomsen <oss@cogitri.dev>

    This file is part of apk-polkit (see https://github.com/Cogitri/apk-polkit).

    apk-polkit is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    apk-polkit is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with apk-polkit.  If not, see <https://www.gnu.org/licenses/>.
*/

module apkd_common.ApkDataBaseOperations;

import apkd_common.CommonOperations;
import std.conv : to;

/// Helper class that aids in translating from a db operation (function)
/// one wants to run to the dbus method or the polkit action.
class ApkDataBaseOperations : CommonOperations
{
    /**
    * All operations that can be done on the database
    */
    enum Enum
    {
        addPackages,
        deletePackages,
        listInstalledPackages,
        listAvailablePackages,
        listUpgradablePackages,
        searchFileOwner,
        searchPackageNames,
        updateRepositories,
        upgradeAllPackages,
        upgradePackages,
    }

    this(Enum val) nothrow
    {
        this.m_val = val;
    }

    this(string methodName)
    {
        this.m_val = methodName.to!Enum;
    }

    override string toString() const
    {
        return this.val.to!string;
    }

    /**
    * Get the polkit action that should be run for an operation
    */
    override string toPolkitAction() const
    {
        immutable auto prefix = "dev.Cogitri.apkPolkit.Helper";

        return prefix ~ "." ~ this.toString();
    }

    @property Enum val() const
    {
        return this.m_val;
    }

private:
    Enum m_val;
}
